'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var fs = require('fs');
var isobject = _interopDefault(require('isobject'));
var pProps = _interopDefault(require('p-props'));
var assert = _interopDefault(require('assert'));
var objectMap = _interopDefault(require('object.map'));

class ConfigError extends Error {
  constructor(errors) {
    super('Configuration could not be loaded.');
    this.errors = errors;
  }

  toJSON() {
    return {
      errors: this.errors.map(error => error.message),
      message: this.message
    };
  }

}

function loadEnvironmentConfig(property) {
  const _property$required = property.required,
        required = _property$required === void 0 ? false : _property$required,
        variableName = property.variableName;
  const value = process.env[variableName];
  const error = required && typeof value !== 'string' && new Error(`${variableName} is not defined.`);
  return {
    error,
    value
  };
}

function loadFileConfig(property) {
  const _property$encoding = property.encoding,
        encoding = _property$encoding === void 0 ? 'utf8' : _property$encoding,
        filePath = property.filePath,
        _property$required = property.required,
        required = _property$required === void 0 ? false : _property$required;
  return new Promise(resolve => fs.readFile(filePath, encoding, (error, value) => {
    var _value;

    return resolve({
      error: required && error,
      value: (_value = value) !== null && _value !== void 0 ? _value : undefined
    });
  }));
}

function parseProperty(result, type, groupName, propertyName) {
  if (type === 'string' || typeof result.value === 'undefined') {
    return result;
  }

  const value = parseInt(result.value, 10);
  const error = Number.isNaN(value) && new Error(`The value for "configMap.${groupName}.${propertyName}" was defined, but could not be coerced to a number.`);
  return {
    error,
    value
  };
}

function loadProperty(property, propertyName, groupName) {
  if (typeof property === 'string') {
    return Promise.resolve(loadEnvironmentConfig({
      variableName: property
    }));
  } // $FlowFixMe


  const isEnvConfig = typeof (property === null || property === void 0 ? void 0 : property.variableName) === 'string'; // $FlowFixMe

  const isFileConfig = typeof (property === null || property === void 0 ? void 0 : property.filePath) === 'string';

  if (!(isEnvConfig || isFileConfig)) {
    return Promise.reject(new Error(`"configMap.${groupName}.${propertyName}" must be a string, EnvironmentConfig object, or FileConfig object.`));
  }

  if (isEnvConfig && isFileConfig) {
    return Promise.reject(new Error(`Cannot determine whether "configMap.${groupName}.${propertyName}" is an EnvironmentConfig object or a FileConfig object.  Both "filePath" and "variableName" are defined.`));
  }

  const _property$type = property.type,
        type = _property$type === void 0 ? 'string' : _property$type;

  if (!['number', 'string'].includes(type)) {
    return Promise.reject(new Error(`"configMap.${groupName}.${propertyName}.type" must be 'number', 'string', or undefined.`));
  }

  return Promise.resolve(isEnvConfig ? loadEnvironmentConfig(property) : loadFileConfig(property)).then(result => parseProperty(result, type, groupName, propertyName));
}

function loadConfig(configMap) {
  if (!isobject(configMap)) {
    return Promise.reject(new Error('"configMap" must be a ConfigMap object.'));
  }

  const errors = [];
  return pProps(configMap, (group, groupName) => {
    if (!isobject(group)) {
      return Promise.reject(new Error(`"configMap.${groupName}" must be a ConfigGroup object.`));
    }

    return pProps(group, (prop, propName) => loadProperty(prop, propName, groupName).then(({
      error,
      value
    }) => {
      if (error) {
        errors.push(error);
      }

      return value;
    }));
  }).then(result => {
    if (errors.length > 0) {
      throw new ConfigError(errors);
    }

    return result;
  });
}

function loadFileConfigSync(property) {
  const _property$encoding = property.encoding,
        encoding = _property$encoding === void 0 ? 'utf8' : _property$encoding,
        filePath = property.filePath,
        _property$required = property.required,
        required = _property$required === void 0 ? false : _property$required;

  try {
    const value = fs.readFileSync(filePath, encoding);
    return {
      value
    };
  } catch (error) {
    return {
      error: required && error
    };
  }
}

function loadPropertySync(property, propertyName, groupName) {
  if (typeof property === 'string') {
    return loadEnvironmentConfig({
      variableName: property
    });
  } // $FlowFixMe


  const isEnvConfig = typeof (property === null || property === void 0 ? void 0 : property.variableName) === 'string'; // $FlowFixMe

  const isFileConfig = typeof (property === null || property === void 0 ? void 0 : property.filePath) === 'string';
  assert(isEnvConfig || isFileConfig, `"configMap.${groupName}.${propertyName}" must be a string, EnvironmentConfig object, or FileConfig object.`);
  assert(!(isEnvConfig && isFileConfig), `Cannot determine whether "configMap.${groupName}.${propertyName}" is an EnvironmentConfig object or a FileConfig object.  Both "filePath" and "variableName" are defined.`);
  const _property$type = property.type,
        type = _property$type === void 0 ? 'string' : _property$type;
  assert(['number', 'string'].includes(type), `"configMap.${groupName}.${propertyName}.type" must be 'number', 'string', or undefined.`);
  const result = isEnvConfig ? loadEnvironmentConfig(property) : loadFileConfigSync(property);
  return parseProperty(result, type, groupName, propertyName);
}

function loadConfigSync(configMap) {
  assert(isobject(configMap), '"configMap" must be a ConfigMap object.');
  const errors = [];
  const result = objectMap(configMap, (group, groupName) => {
    assert(isobject(group), `"configMap.${groupName}" must be a ConfigGroup object.`);
    return objectMap(group, (prop, propName) => {
      const _loadPropertySync = loadPropertySync(prop, propName, groupName),
            error = _loadPropertySync.error,
            value = _loadPropertySync.value;

      if (error) {
        errors.push(error);
      }

      return value;
    });
  });

  if (errors.length > 0) {
    throw new ConfigError(errors);
  }

  return result;
}

exports.loadConfig = loadConfig;
exports.loadConfigSync = loadConfigSync;
//# sourceMappingURL=index.cjs.js.map
